@@ -93,7 +93,7 @@
     "\n")))
 
 (defun riece-handle-301-message (prefix number name string)
-  (if (string-match (concat "^\\(" riece-user-regexp "\\) :?") string)
+  (if (string-match (concat "^\\([^ ]+\\) :?") string)
       (let ((user (match-string 1 string))
 	    (message (substring string (match-end 0))))
 	(riece-user-toggle-away user t)
@@ -127,8 +127,7 @@
 
 (defun riece-handle-311-message (prefix number name string)
   (if (string-match
-       (concat "^\\(" riece-user-regexp
-	       "\\) \\([^ ]+\\) \\([^ ]+\\) \\* :?")
+       (concat "^\\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\) \\* :?")
        string)
       (let ((user (match-string 1 string))
 	    (name (substring string (match-end 0)))
@@ -148,7 +147,7 @@
 
 (defun riece-handle-312-message (prefix number name string)
   (if (string-match
-       (concat "^\\(" riece-user-regexp "\\) \\([^ ]+\\) :?")
+       (concat "^\\([^ ]+\\) \\([^ ]+\\) :?")
        string)
       (riece-insert-info
        (list riece-dialogue-buffer riece-others-buffer)
@@ -174,23 +173,35 @@
 
 (defun riece-handle-317-message (prefix number name string)
   (if (string-match
-       (concat "^\\(" riece-user-regexp "\\) \\([0-9]+\\) ")
+       (concat "^\\([^ ]+\\) \\([0-9]+\\) ")
        string)
-      (let ((user (match-string 1 string))
-	    (idle (match-string 2 string)))
+      (let* ((user (match-string 1 string))
+	     (seconds (string-to-number (match-string 2 string)))
+	     (units (list (cons (/ seconds 60 60 24) "days")
+			  (cons (mod (/ seconds 60 60) 24) "hours")
+			  (cons (mod (/ seconds 60) 60) "minutes")
+			  (cons (mod seconds 60) "seconds"))))
 	(riece-insert-info
 	 (list riece-dialogue-buffer riece-others-buffer)
 	 (concat
 	  (riece-concat-server-name
-	   (format "%s is %s seconds idle"
+	   (format "%s is %s idle"
 		   (riece-format-identity
 		    (riece-make-identity user riece-server-name)
 		    t)
-		   idle))
+		   (mapconcat #'identity
+			      (delq nil
+				    (mapcar
+				     (lambda (unit)
+				       (if (/= (car unit) 0)
+					   (format "%d %s"
+						   (car unit) (cdr unit))))
+				     units))
+			      " ")))
 	  "\n")))))
 
 (defun riece-handle-319-message (prefix number name string)
-  (if (string-match (concat "^\\(" riece-user-regexp "\\) :?") string)
+  (if (string-match (concat "^\\([^ ]+\\) :?") string)
       (let ((user (match-string 1 string))
 	    (channels
 	     (mapconcat
@@ -235,7 +246,7 @@
   (make-local-variable 'riece-353-message-alist)      
   (if (string-match "^[=\*@] *\\([^ ]+\\) +:?" string)
       (let* ((channel (match-string 1 string))
-	     (entry (assoc channel riece-353-message-alist)))
+	     (entry (riece-identity-assoc channel riece-353-message-alist t)))
 	(if entry
 	    (setcdr entry
 		    (concat (cdr entry)
@@ -414,7 +425,7 @@
 	     (channel-identity (riece-make-identity channel
 						    riece-server-name))
 	     (buffer (riece-channel-buffer channel-identity))
-	     (entry (assoc channel riece-353-message-alist))
+	     (entry (riece-identity-assoc channel riece-353-message-alist t))
 	     (string (cdr entry))
 	     (start 0)
 	     users)
