@@ -43,6 +43,7 @@ Local to the buffer in `riece-buffer-list'.")
 
 (defvar riece-update-indicator-functions
   '(riece-update-status-indicators
+    riece-update-channel-status-indicator
     riece-update-channel-indicator
     riece-update-long-channel-indicator
     riece-update-channel-list-indicator)
@@ -70,6 +71,7 @@ Local to the buffer in `riece-buffer-list'.")
    'channel-switched
    (lambda (signal handback)
      (riece-update-status-indicators)
+     (riece-update-channel-status-indicator)
      (riece-update-channel-indicator)
      (riece-update-long-channel-indicator)
      (force-mode-line-update t)
@@ -123,11 +125,13 @@ Local to the buffer in `riece-buffer-list'.")
 		 (riece-identity-server riece-current-channel))
 	  (riece-with-server-buffer (riece-identity-server
 				     riece-current-channel)
-	    (riece-identity-assoc
-	     (riece-identity-prefix (nth 1 (riece-signal-args signal)))
-	     (riece-channel-get-users (riece-identity-prefix
-				       riece-current-channel))
-	     t)))))
+	    (when (riece-channel-p (riece-identity-prefix
+				    riece-current-channel))
+	      (riece-identity-assoc
+	       (riece-identity-prefix (nth 1 (riece-signal-args signal)))
+	       (riece-channel-get-users (riece-identity-prefix
+					 riece-current-channel))
+	       t))))))
   (riece-connect-signal
    'user-renamed
    (lambda (signal handback)
@@ -199,6 +203,7 @@ Local to the buffer in `riece-buffer-list'.")
   (riece-connect-signal
    'channel-operators-changed
    (lambda (signal handback)
+     (riece-update-channel-status-indicator)
      (riece-emit-signal 'user-list-changed riece-current-channel))
    (lambda (signal)
      (and riece-current-channel
@@ -207,6 +212,7 @@ Local to the buffer in `riece-buffer-list'.")
   (riece-connect-signal
    'channel-speakers-changed
    (lambda (signal handback)
+     (riece-update-channel-status-indicator)
      (riece-emit-signal 'user-list-changed riece-current-channel))
    (lambda (signal)
      (and riece-current-channel
@@ -321,24 +327,30 @@ Local to the buffer in `riece-buffer-list'.")
 	(while pointer
 	  (if (cdr pointer)
 	      (setcdr pointer (cons "," (cdr pointer))))
-	  (setq pointer (cdr (cdr pointer)))))
+	  (setq pointer (cdr (cdr pointer))))
+	(setq riece-channel-list-indicator
+	      (riece-normalize-modeline-string riece-channel-list-indicator)))
     (setq riece-channel-list-indicator "No channel")))
 
 (defun riece-update-status-indicators ()
-  (if riece-current-channel
-      (with-current-buffer riece-command-buffer
-	(riece-with-server-buffer (riece-identity-server riece-current-channel)
-	  (setq riece-away-indicator
-		(if (and riece-real-nickname
-			 (riece-user-get-away riece-real-nickname))
-		    "A"
-		  "-")
-		riece-operator-indicator
-		(if (and riece-real-nickname
-			 (riece-user-get-operator riece-real-nickname))
-		    "O"
-		  "-")
-		riece-user-indicator riece-real-nickname))))
+  (let ((server-name (riece-current-server-name)))
+    (if server-name
+	(with-current-buffer riece-command-buffer
+	  (riece-with-server-buffer server-name
+	    (setq riece-away-indicator
+		  (if (and riece-real-nickname
+			   (riece-user-get-away riece-real-nickname))
+		      "A"
+		    "-")
+		  riece-operator-indicator
+		  (if (and riece-real-nickname
+			   (riece-user-get-operator riece-real-nickname))
+		      "O"
+		    "-")
+		  riece-user-indicator
+		  (riece-format-identity
+		   (riece-make-identity riece-real-nickname riece-server-name)
+		   t))))))
   (walk-windows
    (lambda (window)
      (with-current-buffer (window-buffer window)
@@ -351,13 +363,37 @@ Local to the buffer in `riece-buffer-list'.")
 		       "F"
 		     "-"))))))))
 
+(defun riece-update-channel-status-indicator ()
+  (if (and riece-current-channel
+	   (riece-channel-p (riece-identity-prefix riece-current-channel)))
+      (let ((users
+	     (riece-with-server-buffer (riece-identity-server
+					riece-current-channel)
+	       (riece-channel-get-users (riece-identity-prefix
+					 riece-current-channel))))
+	    (nickname
+	     (riece-with-server-buffer (riece-identity-server
+					riece-current-channel)
+	       riece-real-nickname)))
+	(with-current-buffer riece-command-buffer
+	  (setq riece-channel-status-indicator
+		(if nickname
+		    (let ((user (cdr (riece-identity-assoc nickname users t))))
+		      (if (memq ?o user)
+			  "@"
+			(if (memq ?v user)
+			    "+"
+			  "-")))
+		  "-"))))))
+
 (defun riece-update-buffers (&optional buffers)
   (unless buffers
     (setq buffers riece-buffer-list))
   (while buffers
-    (save-excursion
-      (set-buffer (car buffers))
-      (run-hooks 'riece-update-buffer-functions))
+    (if (buffer-live-p (car buffers))
+	(save-excursion
+	  (set-buffer (car buffers))
+	  (run-hooks 'riece-update-buffer-functions)))
     (setq buffers (cdr buffers)))
   (run-hooks 'riece-update-indicator-functions)
   (force-mode-line-update t)
@@ -442,7 +478,8 @@ Local to the buffer in `riece-buffer-list'.")
     (if pointer
 	(setcar pointer nil))
     (if (riece-identity-equal identity riece-current-channel)
-	(riece-switch-to-nearest-channel pointer))))
+	(riece-switch-to-nearest-channel pointer))
+    (funcall riece-buffer-dispose-function (riece-channel-buffer identity))))
 
 (defun riece-redisplay-buffers (&optional force)
   (riece-update-buffers)
