@@ -95,6 +95,19 @@
   (add-text-properties 0 (length string) properties string)
   string)
 
+(defun riece-normalize-modeline-string-1 (string)
+  (if string
+      (if (listp (car string))
+	  (cons (car (car string)) (riece-normalize-modeline-string-1
+				    (append (cdr (car string)) (cdr string))))
+	(cons (car string) (riece-normalize-modeline-string-1
+			    (cdr string))))))
+
+(defun riece-normalize-modeline-string (string)
+  (if (listp string)
+      (list (apply #'concat (riece-normalize-modeline-string-1 string)))
+    string))
+
 (defun riece-put-text-property-nonsticky (start end prop value
 						     &optional object)
   (add-text-properties start end
@@ -103,6 +116,25 @@
 
 (defalias 'riece-facep 'facep)
 
+;;; stolen (and renamed) from emacsbug.el.
+(defun riece-recent-messages (n)
+  "Return N most recent messages, most recent first.
+If N is nil, all messages will be returned."
+  (let ((message-buf (get-buffer "*Messages*")))
+    (if message-buf
+	(with-temp-buffer
+	  (let (beg-pos end-pos)
+	    (with-current-buffer message-buf
+	      (setq end-pos (goto-char (point-max)))
+	      (if n
+		  (progn
+		    (forward-line (- n))
+		    (setq beg-pos (point)))
+		(setq beg-pos (point-min))))
+	    (insert-buffer-substring message-buf beg-pos end-pos)
+	    (reverse-region (point-min) (point-max))
+	    (buffer-string))))))
+
 (provide 'riece-emacs)
 
 ;;; riece-emacs.el ends here
