@@ -70,24 +70,69 @@
 (defun riece-naming-assert-channel-users (users channel-name)
   (let ((channel-identity (riece-make-identity channel-name
 					       riece-server-name))
-	user-identity-list)
-    (while users
-      (riece-user-toggle-channel (car (car users)) channel-name t)
-      (riece-channel-toggle-user channel-name (car (car users)) t)
-      (if (memq ?o (cdr (car users)))
-	  (riece-channel-toggle-operator channel-name (car (car users)) t)
-	(if (memq ?v (cdr (car users)))
-	    (riece-channel-toggle-speaker channel-name (car (car users)) t)
-	  (riece-channel-toggle-operator channel-name (car (car users)) nil)
-	  (riece-channel-toggle-speaker channel-name (car (car users)) nil)))
-      (setq user-identity-list
-	    (cons (cons (riece-make-identity (car (car users))
-					     riece-server-name)
-			(cdr (car users)))
-		  user-identity-list)
-	    users (cdr users)))
+	(pointer users))
+    (while pointer
+      (riece-user-toggle-channel (car (car pointer)) channel-name t)
+      (riece-channel-toggle-user channel-name (car (car pointer)) t)
+      (if (memq ?o (cdr (car pointer)))
+	  (riece-channel-toggle-operator channel-name (car (car pointer)) t)
+	(if (memq ?v (cdr (car pointer)))
+	    (riece-channel-toggle-speaker channel-name (car (car pointer)) t)
+	  (riece-channel-toggle-operator channel-name (car (car pointer)) nil)
+	  (riece-channel-toggle-speaker channel-name (car (car pointer)) nil)))
+      (setq pointer (cdr pointer)))
+    ;; Remove nonexistent users.
+    (setq pointer (riece-channel-users (riece-get-channel channel-name)))
+    (while pointer
+      (unless (assoc (car (car pointer)) users)
+	(riece-user-toggle-channel (car (car pointer)) channel-name nil)
+	(riece-channel-toggle-user channel-name (car (car pointer)) nil))
+      (setq pointer (cdr pointer)))
     (riece-emit-signal 'user-list-changed channel-identity)))
 
+(defun riece-naming-assert-channel-modes (channel modes)
+  (while modes
+    (cond
+     ((eq (riece-mode-flag (car (car modes))) ?o)
+      (riece-channel-toggle-operator channel
+				     (riece-mode-parameter (car (car modes)))
+				     (nth 1 (car modes)))
+      (riece-emit-signal 'channel-operators-changed
+			 (riece-make-identity channel
+					      riece-server-name)
+			 (riece-make-identity (riece-mode-parameter
+					       (car (car modes)))
+					      riece-server-name)
+			 (nth 1 (car modes))))
+     ((eq (riece-mode-flag (car (car modes))) ?v)
+      (riece-channel-toggle-speaker channel
+				    (riece-mode-parameter (car (car modes)))
+				    (nth 1 (car modes)))
+      (riece-emit-signal 'channel-speakers-changed
+			 (riece-make-identity channel
+					      riece-server-name)
+			 (riece-make-identity (riece-mode-parameter
+					       (car (car modes)))
+					      riece-server-name)
+			 (nth 1 (car modes))))
+     ((eq (riece-mode-flag (car (car modes))) ?b)
+      (riece-channel-toggle-banned channel
+				   (riece-mode-parameter (car (car modes)))
+				   (nth 1 (car modes))))
+     ((eq (riece-mode-flag (car (car modes))) ?e)
+      (riece-channel-toggle-uninvited channel
+				      (riece-mode-parameter (car (car modes)))
+				      (nth 1 (car modes))))
+     ((eq (riece-mode-flag (car (car modes))) ?I)
+      (riece-channel-toggle-invited channel
+				    (riece-mode-parameter (car (car modes)))
+				    (nth 1 (car modes))))
+     (t
+      (apply #'riece-channel-toggle-mode channel (car modes))))
+    (setq modes (cdr modes)))
+  (riece-emit-signal 'channel-modes-changed
+		     (riece-make-identity channel riece-server-name)))
+
 (provide 'riece-naming)
 
 ;;; riece-naming.el ends here
