@@ -114,7 +114,12 @@ way is to put Riece variables on .emacs or file loaded from there."
 			  riece-button
 			  riece-menu
 			  riece-icon
-			  riece-ignore)
+			  riece-ignore
+			  riece-log
+			  riece-toolbar
+			  riece-alias
+			  riece-ctlseq
+			  riece-keyword)
   "Add-ons insinuated into Riece."
   :type '(repeat symbol)
   :group 'riece-options)
@@ -194,8 +199,7 @@ way is to put Riece variables on .emacs or file loaded from there."
 		(const :tag "No" nil))
   :group 'riece-server)
 
-(defcustom riece-username (or (getenv "IRCNAME")
-			      (user-real-login-name))
+(defcustom riece-username nil
   "Your user name."
   :type 'string
   :group 'riece-server)
@@ -269,6 +273,12 @@ way is to put Riece variables on .emacs or file loaded from there."
   :type '(string :tag "Quit message")
   :group 'riece-options)
 
+(defcustom riece-part-message nil
+  "Default part message."
+  :type '(choice (const :tag "No message" nil)
+		 (string :tag "Part message"))
+  :group 'riece-options)
+
 (defcustom riece-away-message "Gone"
   "Default away message."
   :type '(string :tag "Away message")
@@ -279,7 +289,7 @@ way is to put Riece variables on .emacs or file loaded from there."
   :type 'boolean
   :group 'riece-options)
 
-(defcustom riece-buffer-dispose-function #'bury-buffer
+(defcustom riece-buffer-dispose-function #'kill-buffer
   "Function called after the buffer was disposed."
   :type 'function
   :group 'riece-options)
@@ -289,12 +299,17 @@ way is to put Riece variables on .emacs or file loaded from there."
   :type 'integer
   :group 'riece-options)
 
-(defcustom riece-max-buffer-size 65535
+(defcustom riece-max-buffer-size nil
   "Maximum size of channel buffers."
   :type '(radio (integer :tag "Number of characters")
 		(const nil))
   :group 'riece-options)
 
+(defcustom riece-shrink-buffer-remove-chars (/ riece-max-send-size 2)
+  "Number of chars removed when shrinking channel buffers."
+  :type 'integer
+  :group 'riece-options)
+
 (defcustom riece-format-time-function #'current-time-string
   "Function to convert the specified time to the human readable form."
   :type 'function
